@@ -76,22 +76,22 @@ def test_init(db):
     User = um.db_adapter.UserClass
 
     # Create user1 with username and email
-    user1 = User(username='user1', email='user1@example.com', password=hashed_password, active=True)
+    user1 = User(username='user1', email='user1@example.com', password=hashed_password, is_enabled=True)
     assert user1
     db.session.add(user1)
 
     # Create user1 with email only
-    user2 = User(email='user2@example.com', password=hashed_password, active=True)
+    user2 = User(email='user2@example.com', password=hashed_password, is_enabled=True)
     assert user2
     db.session.add(user2)
 
     # Create user3 with username and email
-    user3 = User(username='user3', email='user3@example.com', password=hashed_password, active=True)
+    user3 = User(username='user3', email='user3@example.com', password=hashed_password, is_enabled=True)
     assert user3
     db.session.add(user3)
 
     # Create user4 with email only
-    user4 = User(email='user4@example.com', password=hashed_password, active=True)
+    user4 = User(email='user4@example.com', password=hashed_password, is_enabled=True)
     assert user4
     db.session.add(user4)
 
@@ -217,7 +217,7 @@ def test_invalid_login_with_username_form(client):
             username='', password=password)
 
     # Test incorrect username
-    client.post_invalid_form(url, 'Incorrect Username and Password',
+    client.post_invalid_form(url, 'Incorrect Username/Email and Password',
             username='Xuser1', password=password)
 
     # Test empty password
@@ -225,7 +225,7 @@ def test_invalid_login_with_username_form(client):
             username=username, password='')
 
     # Test incorrect password
-    client.post_invalid_form(url, 'Incorrect Username and Password',
+    client.post_invalid_form(url, 'Incorrect Username/Email and Password',
             username=username, password='XPassword1')
 
 def test_invalid_login_with_email_form(client):
@@ -262,6 +262,7 @@ def test_invalid_change_username_form(client):
     # Set user manager config
     um =  current_app.user_manager
     um.enable_username = True
+    um.enable_email = False
 
     # Set default values
     username = 'user1'
@@ -407,8 +408,8 @@ def test_invalid_roles(client):
     assert response_has_string(response, 'You do not have permission to access')
     client.logout()
 
-def test_login_without_confirm(client):
-    print("test_login_without_confirm")
+def test_login_without_confirm_email(client):
+    print("test_login_without_confirm_email")
 
     um = current_app.user_manager
     um.enable_username = False
@@ -434,12 +435,12 @@ def test_login_without_confirm(client):
     User = um.db_adapter.UserClass
     user = User.query.filter(User.email==email).first()
     assert(user)
-    user.active = False
+    user.is_enabled = False
     user.confirmed_at = datetime.utcnow()
 
     # Try logging in into  disabled account
     client.post_invalid_form(url_for('user.login'),
-            'Your account has been disabled.',
+            'Your account has not been enabled.',
             email=email,
             password=password)
 
@@ -475,7 +476,7 @@ def run_all_tests(client):
     test_valid_roles(client)
     test_invalid_roles(client)
 
-    test_login_without_confirm(client)
+    test_login_without_confirm_email(client)
 
     test_cleanup(client.db)
 
