@@ -6,18 +6,15 @@
 
 from datetime import datetime
 from flask import current_app, flash, redirect, render_template, request, url_for
-from flask.ext.login import current_user, login_user, logout_user
-from .decorators import login_required
-from . import emails
-from . import signals
-from .translations import gettext as _
-import sys
-
-# Handle Python 2.x and Python 3.x
-try:
+from flask_login import current_user, login_user, logout_user
+try: # Handle Python 2.x and Python 3.x
     from urllib.parse import quote      # Python 3.x
 except ImportError:
     from urllib import quote            # Python 2.x
+from .decorators import confirm_email_required, login_required
+from . import emails
+from . import signals
+from .translations import gettext as _
 
 def confirm_email(token):
     """ Verify email confirmation token and activate the user account."""
@@ -51,7 +48,7 @@ def confirm_email(token):
             user.confirmed_at = datetime.utcnow()
 
     if user:
-        user.active = True
+        user.set_active(True)
         db_adapter.commit()
     else:                                               # pragma: no cover
         flash(_('Invalid confirmation token.'), 'error')
@@ -63,15 +60,16 @@ def confirm_email(token):
     # Prepare one-time system message
     flash(_('Your email has been confirmed.'), 'success')
 
-    # Redirect to login page or auto-login if configured
+    # Auto-login after confirm or redirect to login page
     next = request.args.get('next', _endpoint_url(user_manager.after_confirm_endpoint))
     if user_manager.auto_login_after_confirm:
-        return _do_login_user(user, next)  # Auto-login user after confirm
+        return _do_login_user(user, next)                       # auto-login
     else:
-        return redirect(url_for('user.login')+'?next='+next)
+        return redirect(url_for('user.login')+'?next='+next)    # redirect to login page
 
 
 @login_required
+@confirm_email_required
 def change_password():
     """ Prompt for old password and new password and change the user's password."""
     user_manager =  current_app.user_manager
@@ -87,7 +85,8 @@ def change_password():
         hashed_password = user_manager.hash_password(form.new_password.data)
 
         # Change password
-        db_adapter.update_object(current_user, password=hashed_password)
+        user_auth = current_user.user_auth if db_adapter.UserAuthClass  and hasattr(current_user, 'user_auth') else current_user
+        db_adapter.update_object(user_auth, password=hashed_password)
         db_adapter.commit()
 
         # Send 'password_changed' email
@@ -107,6 +106,7 @@ def change_password():
     return render_template(user_manager.change_password_template, form=form)
 
 @login_required
+@confirm_email_required
 def change_username():
     """ Prompt for new username and old password and change the user's username."""
     user_manager =  current_app.user_manager
@@ -121,7 +121,8 @@ def change_username():
         new_username = form.new_username.data
 
         # Change username
-        db_adapter.update_object(current_user, username=new_username)
+        user_auth = current_user.user_auth if db_adapter.UserAuthClass and hasattr(current_user, 'user_auth') else current_user
+        db_adapter.update_object(user_auth, username=new_username)
         db_adapter.commit()
 
         # Send 'username_changed' email
@@ -141,6 +142,7 @@ def change_username():
     return render_template(user_manager.change_username_template, form=form)
 
 @login_required
+@confirm_email_required
 def email_action(id, action):
     """ Perform action 'action' on UserEmail object 'id'
     """
@@ -243,33 +245,28 @@ def login():
     # Process valid POST
     if request.method=='POST' and login_form.validate():
         # Retrieve User
+        user = None
         user_email = None
         if user_manager.enable_username:
-            # Find user by username or email address
+            # Find user record by username
             user = user_manager.find_user_by_username(login_form.username.data)
             user_email = None
+            # Find primary user_email record
             if user and db_adapter.UserEmailClass:
                 user_email = db_adapter.find_first_object(db_adapter.UserEmailClass,
                         user_id=int(user.get_id()),
                         is_primary=True,
                         )
+            # Find user record by email (with form.username)
             if not user and user_manager.enable_email:
                 user, user_email = user_manager.find_user_by_email(login_form.username.data)
         else:
-            # Find user by email address
+            # Find user by email (with form.email)
             user, user_email = user_manager.find_user_by_email(login_form.email.data)
 
         if user:
-            if user.active:
-                return _do_login_user(user, login_form.next.data, login_form.remember_me.data)  # Login user after Login
-            else:
-                confirmed_at = user_email.confirmed_at if user_email else user.confirmed_at
-                if user_manager.enable_confirm_email and not confirmed_at:
-                    url = url_for('user.resend_confirm_email')
-                    flash(_('Your email address has not yet been confirmed. Check your email Inbox and Spam folders for the confirmation email or <a href="%(url)s">Re-send confirmation email</a>.', url=url), 'error')
-                else:
-                    flash(_('Your account has been disabled.'), 'error')
-                return redirect(url_for('user.home'))
+            # Log user in
+            return _do_login_user(user, login_form.next.data, login_form.remember_me.data)
 
     # Process GET or invalid POST
     return render_template(user_manager.login_template,
@@ -296,6 +293,7 @@ def logout():
 
 
 @login_required
+@confirm_email_required
 def manage_emails():
     user_manager =  current_app.user_manager
     db_adapter = user_manager.db_adapter
@@ -329,53 +327,72 @@ def register():
     login_form = user_manager.login_form()                      # for login_or_register.html
     register_form = user_manager.register_form(request.form)    # for register.html
     if request.method!='POST':
-        login_form.next.data    = register_form.next.data = next
+        login_form.next.data     = register_form.next.data     = next
         login_form.reg_next.data = register_form.reg_next.data = reg_next
 
     # Process valid POST
     if request.method=='POST' and register_form.validate():
 
         # Create a User object using Form fields that have a corresponding User field
-        User = user_manager.db_adapter.UserClass
+        User = db_adapter.UserClass
         user_class_fields = User.__dict__
         user_fields = {}
 
         # Create a UserEmail object using Form fields that have a corresponding UserEmail field
-        if user_manager.db_adapter.UserEmailClass:
-            UserEmail = user_manager.db_adapter.UserEmailClass
+        if db_adapter.UserEmailClass:
+            UserEmail = db_adapter.UserEmailClass
             user_email_class_fields = UserEmail.__dict__
             user_email_fields = {}
 
-        # Create a UserProfile object using Form fields that have a corresponding UserProfile field
-        if user_manager.db_adapter.UserProfileClass:
-            UserProfile = user_manager.db_adapter.UserProfileClass
-            user_profile_class_fields = UserProfile.__dict__
-            user_profile_fields = {}
+        # Create a UserAuth object using Form fields that have a corresponding UserAuth field
+        if db_adapter.UserAuthClass:
+            UserAuth = db_adapter.UserAuthClass
+            user_auth_class_fields = UserAuth.__dict__
+            user_auth_fields = {}
 
-        # User.active is True if not USER_ENABLE_CONFIRM_EMAIL and False otherwise
-        user_fields['active'] = not user_manager.enable_confirm_email
+        # Enable user account
+        if db_adapter.UserProfileClass:
+            if hasattr(db_adapter.UserProfileClass, 'active'):
+                user_auth_fields['active'] = True
+            elif hasattr(db_adapter.UserProfileClass, 'is_enabled'):
+                user_auth_fields['is_enabled'] = True
+            else:
+                user_auth_fields['is_active'] = True
+        else:
+            if hasattr(db_adapter.UserClass, 'active'):
+                user_fields['active'] = True
+            elif hasattr(db_adapter.UserClass, 'is_enabled'):
+                user_fields['is_enabled'] = True
+            else:
+                user_fields['is_active'] = True
 
         # For all form fields
         for field_name, field_value in register_form.data.items():
             # Hash password field
             if field_name=='password':
-                user_fields['password'] = user_manager.hash_password(register_form.password.data)
+                hashed_password = user_manager.hash_password(field_value)
+                if db_adapter.UserAuthClass:
+                    user_auth_fields['password'] = hashed_password
+                else:
+                    user_fields['password'] = hashed_password
             # Store corresponding Form fields into the User object and/or UserProfile object
             else:
                 if field_name in user_class_fields:
                     user_fields[field_name] = field_value
-                if user_manager.db_adapter.UserEmailClass:
+                if db_adapter.UserEmailClass:
                     if field_name in user_email_class_fields:
                         user_email_fields[field_name] = field_value
-                if user_manager.db_adapter.UserProfileClass:
-                    if field_name in user_profile_class_fields:
-                        user_profile_fields[field_name] = field_value
+                if db_adapter.UserAuthClass:
+                    if field_name in user_auth_class_fields:
+                        user_auth_fields[field_name] = field_value
 
         # Add User record using named arguments 'user_fields'
         user = db_adapter.add_object(User, **user_fields)
+        if db_adapter.UserProfileClass:
+            user_profile = user
 
         # Add UserEmail record using named arguments 'user_email_fields'
-        if user_manager.db_adapter.UserEmailClass:
+        if db_adapter.UserEmailClass:
             user_email = db_adapter.add_object(UserEmail,
                     user=user,
                     is_primary=True,
@@ -383,9 +400,13 @@ def register():
         else:
             user_email = None
 
-        # Add UserProfile record using named arguments 'user_profile_fields'
-        if user_manager.db_adapter.UserProfileClass:
-            user.user_profile = db_adapter.add_object(UserProfile, **user_profile_fields)
+        # Add UserAuth record using named arguments 'user_auth_fields'
+        if db_adapter.UserAuthClass:
+            user_auth = db_adapter.add_object(UserAuth, **user_auth_fields)
+            if db_adapter.UserProfileClass:
+                user = user_auth
+            else:
+                user.user_auth = user_auth
 
         db_adapter.commit()
 
@@ -403,13 +424,17 @@ def register():
         # Send user_registered signal
         signals.user_registered.send(current_app._get_current_object(), user=user)
 
-        # Redirect to login page or auto-login if configured
-        reg_next = register_form.reg_next.data
+        # Redirect if USER_ENABLE_CONFIRM_EMAIL is set
         if user_manager.enable_confirm_email:
-            return redirect(reg_next)
+            next = request.args.get('next', _endpoint_url(user_manager.after_register_endpoint))
+            return redirect(next)
+
+        # Auto-login after register or redirect to login page
+        next = request.args.get('next', _endpoint_url(user_manager.after_confirm_endpoint))
         if user_manager.auto_login_after_register:
-            return _do_login_user(user, reg_next)  # Auto-login user after Register
-        return redirect(_endpoint_url(user_manager.after_register_endpoint))
+            return _do_login_user(user, reg_next)                     # auto-login
+        else:
+            return redirect(url_for('user.login')+'?next='+reg_next)  # redirect to login page
 
     # Process GET or invalid POST
     return render_template(user_manager.register_template,
@@ -482,7 +507,8 @@ def reset_password(token):
 
         # Change password
         hashed_password = user_manager.hash_password(form.new_password.data)
-        db_adapter.update_object(user, password=hashed_password)
+        user_auth = user.user_auth if db_adapter.UserAuthClass and hasattr(user, 'user_auth') else user
+        db_adapter.update_object(user_auth, password=hashed_password)
         db_adapter.commit()
 
         # Send 'password_changed' email
@@ -492,15 +518,30 @@ def reset_password(token):
         # Prepare one-time system message
         flash(_("Your password has been reset successfully. Please sign in with your new password"), 'success')
 
-        # Redirect to the login page
-        return redirect(url_for('user.login'))
+        # Auto-login after reset password or redirect to login page
+        next = request.args.get('next', _endpoint_url(user_manager.after_reset_password_endpoint))
+        if user_manager.auto_login_after_reset_password:
+            return _do_login_user(user, next)                       # auto-login
+        else:
+            return redirect(url_for('user.login')+'?next='+next)    # redirect to login page
 
     # Process GET or invalid POST
     return render_template(user_manager.reset_password_template, form=form)
 
 
+def unconfirmed():
+    """ Prepare a Flash message and redirect to USER_UNCONFIRMED_URL"""
+    # Prepare Flash message
+    url = request.script_root + request.path
+    flash(_("You must confirm your email to access '%(url)s'.", url=url), 'error')
+
+    # Redirect to USER_UNCONFIRMED_EMAIL_ENDPOINT
+    user_manager = current_app.user_manager
+    return redirect(_endpoint_url(user_manager.unconfirmed_email_endpoint))
+
+
 def unauthenticated():
-    """ Prepare a Flash message and redirect to USER_UNAUTHENTICATED_URL"""
+    """ Prepare a Flash message and redirect to USER_UNAUTHENTICATED_ENDPOINT"""
     # Prepare Flash message
     url = request.url
     flash(_("You must be signed in to access '%(url)s'.", url=url), 'error')
@@ -508,23 +549,24 @@ def unauthenticated():
     # quote the fully qualified url
     quoted_url = quote(url)
 
-    # Redirect to USER_UNAUTHENTICATED_URL
+    # Redirect to USER_UNAUTHENTICATED_ENDPOINT
     user_manager = current_app.user_manager
     return redirect(_endpoint_url(user_manager.unauthenticated_endpoint)+'?next='+ quoted_url)
 
 
 def unauthorized():
-    """ Prepare a Flash message and redirect to USER_UNAUTHORIZED_URL"""
+    """ Prepare a Flash message and redirect to USER_UNAUTHORIZED_ENDPOINT"""
     # Prepare Flash message
     url = request.script_root + request.path
     flash(_("You do not have permission to access '%(url)s'.", url=url), 'error')
 
-    # Redirect to USER_UNAUTHORIZED_URL
+    # Redirect to USER_UNAUTHORIZED_ENDPOINT
     user_manager = current_app.user_manager
     return redirect(_endpoint_url(user_manager.unauthorized_endpoint))
 
 
 @login_required
+@confirm_email_required
 def user_profile():
     user_manager = current_app.user_manager
     return render_template(user_manager.user_profile_template)
@@ -585,21 +627,36 @@ def _send_confirm_email(user, user_email):
 
 
 def _do_login_user(user, next, remember_me=False):
-    if user and user.is_active:
-        # Use Flask-Login to sign in user
-        #print('login_user: remember_me=', remember_me)
-        login_user(user, remember=remember_me)
+    # User must have been authenticated
+    if not user: return unauthenticated()
 
-        # Send user_logged_in signal
-        signals.user_logged_in.send(current_app._get_current_object(), user=user)
+    # Check if user account has been disabled
+    if not user.is_active():
+        flash(_('Your account has not been enabled.'), 'error')
+        return redirect(url_for('user.home'))
 
-        # Prepare one-time system message
-        flash(_('You have signed in successfully.'), 'success')
+    # Check if user has a confirmed email address
+    user_manager = current_app.user_manager
+    if user_manager.enable_email and user_manager.enable_confirm_email \
+            and not current_app.user_manager.enable_login_without_confirm_email \
+            and not user.has_confirmed_email():
+        url = url_for('user.resend_confirm_email')
+        flash(_('Your email address has not yet been confirmed. Check your email Inbox and Spam folders for the confirmation email or <a href="%(url)s">Re-send confirmation email</a>.', url=url), 'error')
+        return redirect(url_for('user.home'))
 
-        # Redirect to 'next' URL
-        return redirect(next)
+    # Use Flask-Login to sign in user
+    #print('login_user: remember_me=', remember_me)
+    login_user(user, remember=remember_me)
+
+    # Send user_logged_in signal
+    signals.user_logged_in.send(current_app._get_current_object(), user=user)
+
+    # Prepare one-time system message
+    flash(_('You have signed in successfully.'), 'success')
+
+    # Redirect to 'next' URL
+    return redirect(next)
 
-    return unauthenticated()
 
 def _endpoint_url(endpoint):
     url = '/'
